package org.processmining.plugins.plg;


import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.plugins.plg.config.BasicPluginConfiguration;
import org.processmining.plugins.plg.generator.log.PLGLogGeneratorConfiguration;
import org.processmining.plugins.plg.generator.process.PLGProcessGeneratorConfiguration;
import org.processmining.plugins.plg.logic.models.distributions.ProbabilityDistribution;

/**
 * Test driver plugin class
 * 
 * @author Andrea Burattin
 */
public class PLGTestDriver {

	/**
	 * This plugin is useful for testing the generation of a single process
	 * generator of single log generator 
	 * 
	 * @param context
	 * @return two configuration objects: one for a process generation and
	 * another one for the log generation
	 */
	@Plugin (
		name = "Test driver - PLG Process Generator",
		parameterLabels = { },
		returnLabels = { "Process configuration", "Log configuration" },
		returnTypes = { PLGProcessGeneratorConfiguration.class, PLGLogGeneratorConfiguration.class },
		userAccessible = false,
		help = "This plugin will generate a dummy configuration for the generator plugin."
	)
	@UITopiaVariant (
		affiliation = BasicPluginConfiguration.AFFILIATION,
		author = BasicPluginConfiguration.AUTHOR,
		email = BasicPluginConfiguration.EMAIL,
		pack = "PLG - Processes and Log Generator"
	)
	public Object[] generateProcess(PluginContext context) {
		return new Object[] {
			new PLGProcessGeneratorConfiguration(
				4, ProbabilityDistribution.uniformDistributionFactory(),
				4, ProbabilityDistribution.uniformDistributionFactory(),
				20, 50, 50, 50, 50,
				3,
				ProbabilityDistribution.uniformDistributionFactory(),
				ProbabilityDistribution.uniformDistributionFactory()),
			new PLGLogGeneratorConfiguration(100, 0, 0)
		};
	}
}
