package org.processmining.plugins.plg.generator.log;

import org.processmining.framework.annotations.AuthoredType;
import org.processmining.plugins.plg.config.BasicPluginConfiguration;

/**
 * Configuration of a single log generator
 * 
 * @author Andrea Burattin
 */
@AuthoredType(
	typeName = "PLG Log configuration",
	affiliation = BasicPluginConfiguration.AFFILIATION,
	author = BasicPluginConfiguration.AUTHOR,
	email = BasicPluginConfiguration.EMAIL
)
public class PLGLogGeneratorConfiguration {

	private int numberOfTraces;
	private int percentOfNoise;
	private int percentOfInterval;
	
	
	/**
	 * @param numberOfTraces
	 * @param percentOfInterval
	 * @param percentOfNoise
	 */
	public PLGLogGeneratorConfiguration(int numberOfTraces,
			int percentOfInterval, int percentOfNoise) {
		setNumberOfTraces(numberOfTraces);
		setPercentOfInterval(percentOfInterval);
		setPercentOfNoise(percentOfNoise);
	}
	
	
	/**
	 * @param percentOfInterval
	 */
	public void setPercentOfInterval(int percentOfInterval) {
		if (percentOfInterval < 0 || percentOfInterval > 100) {
			percentOfInterval = 0;
		}
		this.percentOfInterval = percentOfInterval;
	}
	
	
	/**
	 * @return
	 */
	public int getPercentOfInterval() {
		return percentOfInterval;
	}
	
	
	/**
	 * @param percentOfNoise
	 */
	public void setPercentOfNoise(int percentOfNoise) {
		if (percentOfNoise < 0 || percentOfNoise > 100) {
			percentOfNoise = 0;
		}
		this.percentOfNoise = percentOfNoise;
	}
	
	
	/**
	 * @return
	 */
	public int getPercentOfNoise() {
		return percentOfNoise;
	}
	
	
	/**
	 * @param numberOfTraces
	 */
	public void setNumberOfTraces(int numberOfTraces) {
		if (numberOfTraces < 1) {
			numberOfTraces = 1;
		}
		this.numberOfTraces = numberOfTraces;
	}
	
	
	/**
	 * @return
	 */
	public int getNumberOfTraces() {
		return numberOfTraces;
	}
}
