package org.processmining.plugins.plg.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.SwingConstants;
import javax.swing.UIManager;
import javax.swing.border.EmptyBorder;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.plaf.basic.BasicTreeUI;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import org.deckfour.uitopia.ui.util.ImageLoader;
import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.framework.util.Pair;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.heuristics.HeuristicsNet;
import org.processmining.plugins.plg.models.PLGLog;
import org.processmining.plugins.plg.models.PLGMineResultCollection;
import org.processmining.plugins.plg.models.PlgProcess;

import com.fluxicon.slickerbox.components.HeaderBar;
import com.fluxicon.slickerbox.components.SlickerTabbedPane;
import com.fluxicon.slickerbox.factory.SlickerDecorator;
import com.fluxicon.slickerbox.factory.SlickerFactory;

/**
 * This class contains the mine result collection viewer
 * 
 * @author Andrea Burattin
 */
@Plugin(
	name = "Mine result collection visualization", 
	returnLabels = { "Visualized mine result collection" }, 
	returnTypes = { JComponent.class }, 
	parameterLabels = { "Mine result collection" }, 
	userAccessible = true
)
@Visualizer
public class MineResultCollectionViewer {
	
	/**
	 * 
	 */
	private class LogListListener implements ActionListener {
		JPanel containerProcess;
		JTree processesLogList;
		UIPluginContext context;
		JLabel messages;
		
		
		/**
		 * Basic constructor
		 * 
		 * @param containerProcess the JPanel with the visualizer
		 * @param processesLogList the component with the list
		 * @param context the visualizer context
		 */
		public LogListListener(JLabel messages, JPanel containerProcess, JTree processesLogList, UIPluginContext context) {
			super();
			this.messages = messages;
			this.processesLogList = processesLogList;
			this.containerProcess = containerProcess;
			this.context = context;
		}
		
		
		@SuppressWarnings("unchecked")
		@Override
		public void actionPerformed(ActionEvent arg0) {
			if (processesLogList.getSelectionCount() == 0) {
				return;
			}
			
			TreePath path = processesLogList.getSelectionPath();
			Object o = ((DefaultMutableTreeNode)path.getLastPathComponent()).getUserObject();
			
			JComponent visualizer = null;
			String title = null;
			
			try {
				
				if (o instanceof PlgProcess) {
					final PlgProcess p = (PlgProcess)o;
					
					messages.setText("Showing " + p.getName() + "... ");

					visualizer = context.tryToFindOrConstructFirstObject(JComponent.class, null, null, p);
					title = p.getName();
					// cleaning the jcomponent of the visualizer plugin from the workspace
					context.getGlobalContext().getResourceManager().getResourceForInstance(visualizer).destroy();

					messages.setText(messages.getText() + "done!");
					
					
				} else if (o instanceof Pair<?, ?>) {
					
					PlgProcess p = (PlgProcess) ((DefaultMutableTreeNode)path.getPathComponent(1)).getUserObject();
					Pair<PLGLog, Object> pair = (Pair<PLGLog, Object>)o;
					PLGLog l = pair.getFirst();
					Object m = pair.getSecond();
					XLog log = l.getLog();
					
					messages.setText("Showing " + l.getId() + "... ");
					
//					JComponent logSummary = context.tryToFindOrConstructFirstNamedObject(JComponent.class, "Log Summary", null, null, log);
					JComponent logSummary = context.tryToFindOrConstructFirstObject(JComponent.class, null, null, log);
					JComponent mineVisualizer = null;
					
					Class<?> classGenetic = null;
					try {
						classGenetic = Class.forName("org.processmining.plugins.heuristicsnet.array.visualization.HeuristicsNetArrayObject");
					} catch (ClassNotFoundException e1) {
						e1.printStackTrace();
					}
					
					/*Class classFuzzy = null;
					try {
						classFuzzy = Class.forName("org.processmining.models.graphbased.directed.fuzzymodel.MutableFuzzyGraph");
					} catch (ClassNotFoundException e1) {
						e1.printStackTrace();
					}*/
					
					if (m instanceof HeuristicsNet) {
						HeuristicsNet hn = (HeuristicsNet)m;
						mineVisualizer = context.tryToFindOrConstructFirstObject(JComponent.class, null, null, hn);
					} else if (m instanceof Petrinet) {
						Petrinet pn = (Petrinet)m;
						mineVisualizer = context.tryToFindOrConstructFirstObject(JComponent.class, null, null, pn);
					} /*else if (classFuzzy.isInstance(m)) {
						mineVisualizer = context.tryToFindOrConstructFirstNamedObject(JComponent.class, "Fuzzy Model visualization", null, null, m);
					}*/ else if (classGenetic.isInstance(m)) {
						mineVisualizer = context.tryToFindOrConstructFirstObject(JComponent.class, null, null, m);
					}
					
					// create the tabbed pane and show the two elements
					visualizer = SlickerFactory.instance().createTabbedPane("", new Color(40,40,40), Color.gray, Color.lightGray);
					((SlickerTabbedPane)visualizer).addTab("Mining result", mineVisualizer);
					((SlickerTabbedPane)visualizer).addTab("Log Summary", logSummary);
					
					// cleaning the jcomponent of the visualizer plugin from the workspace
					context.getGlobalContext().getResourceManager().getResourceForInstance(logSummary).destroy();
					context.getGlobalContext().getResourceManager().getResourceForInstance(mineVisualizer).destroy();
					
					title = p.getName() + " -> " + l.getId();
					
					messages.setText(messages.getText() + "done!");
					
				} else {
					return;
				}
			
			} catch (ConnectionCannotBeObtained e) {
				e.printStackTrace();
			}
			
			containerProcess.removeAll();
			containerProcess.add(GUIUtils.prepareTitle(title), BorderLayout.NORTH);
			containerProcess.add(visualizer, BorderLayout.CENTER);
			containerProcess.validate();
		}
	}
	
	
	@PluginVariant(
		requiredParameterLabels = {0}
	)
	public JComponent visualize(UIPluginContext context, PLGMineResultCollection collection) {
		JPanel containerMain = SlickerFactory.instance().createGradientPanel(new Color(60,60,60), new Color(20,20,20));
		JPanel containerBody = new JPanel();
		JPanel containerBodyLeft = new JPanel();
		JPanel containerBodyCenter = new JPanel();
		JPanel containerList = SlickerFactory.instance().createRoundedPanel(15, Color.gray);
		JButton buttonReloadView = SlickerFactory.instance().createButton("Show");
		JPanel containerProcess = SlickerFactory.instance().createRoundedPanel(15, Color.gray);
		HeaderBar title = new HeaderBar("Mining results");
		final JTree processesLogList;
		final JLabel messages = GUIUtils.prepareLabel(" ");
		
		messages.setBorder(new EmptyBorder(0, 0, 0, 0));
		messages.setOpaque(false);
		messages.setForeground(Color.gray);
		messages.setHorizontalAlignment(SwingConstants.RIGHT);
		
		containerList.setLayout(new BorderLayout());
		containerProcess.setLayout(new BorderLayout());
		
		containerBody.setOpaque(false);
		containerBodyLeft.setOpaque(false);
		containerBodyLeft.setLayout(new BorderLayout());
		((BorderLayout)containerBodyLeft.getLayout()).setVgap(5);
		containerBodyLeft.setBorder(new EmptyBorder(25, 25, 25, 25));
		
		containerBodyCenter.setOpaque(false);
		containerBodyCenter.setLayout(new BorderLayout());
		containerBodyCenter.setBorder(new EmptyBorder(25, 0, 25, 25));
		
		containerBody.setLayout(new BorderLayout());
		containerBody.add(containerBodyLeft, BorderLayout.WEST);
		containerBody.add(containerBodyCenter, BorderLayout.CENTER);
		
		containerBodyLeft.add(containerList, BorderLayout.CENTER);
		containerBodyLeft.add(messages, BorderLayout.SOUTH);
		
		containerBodyCenter.add(containerProcess);
		
		// list of logs and processes
		DefaultMutableTreeNode root = new DefaultMutableTreeNode("Mine collection");
		DefaultTreeModel processesListModel = new DefaultTreeModel(root);
		for (PlgProcess p : collection.keySet()) {
			DefaultMutableTreeNode nodeProcess = new DefaultMutableTreeNode(p);
			root.add(nodeProcess);
			for (Pair<PLGLog, Object> l : collection.get(p)) {
				DefaultMutableTreeNode nodeLog = new DefaultMutableTreeNode(l);
				nodeProcess.add(nodeLog);
			}
		}
		
		UIManager.put("Tree.hash", Color.darkGray);
		
		processesLogList = new JTree(processesListModel);

		((BasicTreeUI)processesLogList.getUI()).setExpandedIcon(null);
		((BasicTreeUI)processesLogList.getUI()).setCollapsedIcon(null);
		
		processesLogList.setForeground(Color.white);
		processesLogList.setBackground(Color.gray);
		processesLogList.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
		processesLogList.expandRow(1);
		
		processesLogList.addTreeSelectionListener(new TreeSelectionListener() {
		    public void valueChanged(TreeSelectionEvent e) {
		        messages.setText(" ");
		    }
		});
		
		DefaultTreeCellRenderer renderer = (DefaultTreeCellRenderer) processesLogList.getCellRenderer();
		renderer.setBackground(Color.gray);
		renderer.setBackgroundNonSelectionColor(Color.gray);
		renderer.setBackgroundSelectionColor(Color.darkGray);
		renderer.setForeground(Color.WHITE);
		renderer.setTextNonSelectionColor(Color.WHITE);
		renderer.setTextSelectionColor(Color.white);
		renderer.setOpenIcon(getProcessImageIcon());
		renderer.setClosedIcon(getProcessImageIcon());
		renderer.setLeafIcon(getLogImageIcon());
		
		JScrollPane processesScrollPane = new JScrollPane(processesLogList);
		SlickerDecorator.instance().decorate(processesScrollPane, Color.gray, Color.black, Color.darkGray);
		processesScrollPane.setBorder(new EmptyBorder(0, 0, 0, 0));
		buttonReloadView.addActionListener(new LogListListener(messages, containerProcess, processesLogList, context));
		
		JPanel tmp0 = new JPanel(); tmp0.setOpaque(false);
		tmp0.setLayout(new BorderLayout());
		tmp0.setBorder(new EmptyBorder(5, 0, 0, 0));
		tmp0.add(buttonReloadView, BorderLayout.EAST);
		
		JLabel tmp1 = GUIUtils.prepareLabel("Double click a process to show its logs:");
		tmp1.setBorder(new EmptyBorder(5, 5, 10, 5));
		containerList.add(tmp1, BorderLayout.NORTH);
		containerList.add(processesScrollPane, BorderLayout.CENTER);
		containerList.add(tmp0, BorderLayout.SOUTH);
		
		// final layout
		
		containerMain.setLayout(new BorderLayout());
		containerMain.add(title, BorderLayout.NORTH);
		containerMain.add(containerBody, BorderLayout.CENTER);
		
		return containerMain;
	}
	
	
	/**
	 * This method returns the correctly scaled image icon for a log
	 *  
	 * @return the image icon for the log
	 */
	private static ImageIcon getLogImageIcon() {
		ImageIcon iconLog = new ImageIcon(ImageLoader.load("resourcetype_log_30x35.png"));
		Image img = iconLog.getImage();  
		Image newimg = img.getScaledInstance(15, 17, java.awt.Image.SCALE_SMOOTH);  
		return new ImageIcon(newimg);
	}
	
	
	/**
	 * This method returns the correctly scaled image icon for a process model 
	 * 
	 * @return the image icon for the process model
	 */
	private static ImageIcon getProcessImageIcon() {
		ImageIcon iconLog = new ImageIcon(ImageLoader.load("resourcetype_model_30x35.png"));
		Image img = iconLog.getImage();  
		Image newimg = img.getScaledInstance(15, 17, java.awt.Image.SCALE_SMOOTH);  
		return new ImageIcon(newimg);
	}
	
}
