package org.processmining.plugins.plg.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.SimpleDateFormat;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.SwingConstants;
import javax.swing.border.EmptyBorder;

import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.plugins.plg.models.PLGProcessCollection;
import org.processmining.plugins.plg.models.PlgProcess;

import com.fluxicon.slickerbox.components.HeaderBar;
import com.fluxicon.slickerbox.factory.SlickerDecorator;
import com.fluxicon.slickerbox.factory.SlickerFactory;

/**
 * This class contains the process models collection viewer
 * 
 * @author Andrea Burattin
 */
@Plugin(
	name = "Process models collection visualization", 
	returnLabels = { "Visualized models collection" }, 
	returnTypes = { JComponent.class }, 
	parameterLabels = { "Process models collection" }, 
	userAccessible = true
)
@Visualizer
public class ProcessCollectionViewer {
	
	/**
	 * This class is a listener for the list of processes. Its role is to show
	 * the correct process into the proper panel, when it is clicked on the
	 * list.
	 */
	private class ProcessesListListener implements ActionListener {
		JPanel containerProcess;
		@SuppressWarnings("rawtypes")
		JList processesList;
		PluginContext context;
		
		@SuppressWarnings("rawtypes")
		public ProcessesListListener(JPanel containerProcess, JList processesList, PluginContext context) {
			super();
			this.processesList = processesList;
			this.containerProcess = containerProcess;
			this.context = context;
		}
		
		@Override
		public void actionPerformed(ActionEvent arg0) {
			PlgProcess p = (PlgProcess)processesList.getSelectedValue();
			if (p != null) {
				containerProcess.removeAll();
				try {
					JComponent processVisualizer = null;
					processVisualizer = context.tryToFindOrConstructFirstObject(JComponent.class, null, null, p);
					
					containerProcess.add(GUIUtils.prepareTitle(p.getName()), BorderLayout.NORTH);
					containerProcess.add(processVisualizer, BorderLayout.CENTER);
					containerProcess.validate();
				} catch (ConnectionCannotBeObtained e) {
					e.printStackTrace();
				}
			}
		}
	}
	

	@SuppressWarnings({ "rawtypes", "unchecked" })
	@PluginVariant(
		requiredParameterLabels = {0}
	)
	public JComponent visualize(PluginContext context, PLGProcessCollection collection) {
		JPanel containerMain = SlickerFactory.instance().createGradientPanel(new Color(60,60,60), new Color(20,20,20));
		JPanel containerBody = new JPanel();
		JPanel containerBodyLeft = new JPanel();
		JPanel containerBodyCenter = new JPanel();
		JPanel containerProperties = SlickerFactory.instance().createRoundedPanel(15, Color.gray);
		JPanel containerList = SlickerFactory.instance().createRoundedPanel(15, Color.gray);
		JButton buttonReloadView = SlickerFactory.instance().createButton("Show process");
		JPanel containerProcess = SlickerFactory.instance().createRoundedPanel(15, Color.gray);
		HeaderBar title = new HeaderBar("Basic collection properties");
		JList processesList;
		
		containerProperties.setLayout(new GridBagLayout());
		containerList.setLayout(new BorderLayout());
		containerProcess.setLayout(new BorderLayout());
		
		containerBody.setOpaque(false);
		containerBodyLeft.setOpaque(false);
		containerBodyLeft.setLayout(new BorderLayout());
		((BorderLayout)containerBodyLeft.getLayout()).setVgap(20);
		containerBodyLeft.setBorder(new EmptyBorder(25, 25, 25, 25));
		
		containerBodyCenter.setOpaque(false);
		containerBodyCenter.setLayout(new BorderLayout());
		containerBodyCenter.setBorder(new EmptyBorder(25, 0, 25, 25));
		
		containerBody.setLayout(new BorderLayout());
		containerBody.add(containerBodyLeft, BorderLayout.WEST);
		containerBody.add(containerBodyCenter, BorderLayout.CENTER);
		
		containerBodyLeft.add(containerProperties, BorderLayout.NORTH);
		containerBodyLeft.add(containerList, BorderLayout.CENTER);
		containerBodyLeft.add(GUIUtils.credits, BorderLayout.SOUTH);
		
		containerBodyCenter.add(containerProcess);
		
		// collection properties
		GridBagConstraints c = new GridBagConstraints();
		c.anchor = GridBagConstraints.EAST;
		c.ipadx = 5; c.ipady = 5;
		c.gridx = 0; c.gridy = 0;
		containerProperties.add(GUIUtils.prepareLabel("Collection cardinality: ", SwingConstants.RIGHT), c);
		
		c = new GridBagConstraints();
		c.anchor = GridBagConstraints.WEST;
		c.ipadx = 5; c.ipady = 5;
		c.gridx = 1; c.gridy = 0;
		containerProperties.add(GUIUtils.prepareLabel(collection.getConfiguration().getInstancesNo()), c);
		
		c = new GridBagConstraints();
		c.anchor = GridBagConstraints.EAST;
		c.ipadx = 5; c.ipady = 5;
		c.gridx = 0; c.gridy = 1;
		containerProperties.add(GUIUtils.prepareLabel("Loop percentage: ", SwingConstants.RIGHT), c);
		
		c = new GridBagConstraints();
		c.anchor = GridBagConstraints.WEST;
		c.ipadx = 5; c.ipady = 5;
		c.gridx = 1; c.gridy = 1;
		containerProperties.add(GUIUtils.prepareLabel(collection.getConfiguration().getSingleProcess().getLoopPercent()), c);
		
		c = new GridBagConstraints();
		c.anchor = GridBagConstraints.EAST;
		c.ipadx = 5; c.ipady = 5;
		c.gridx = 0; c.gridy = 2;
		containerProperties.add(GUIUtils.prepareLabel("Single activity percentage: ", SwingConstants.RIGHT), c);
		
		c = new GridBagConstraints();
		c.anchor = GridBagConstraints.WEST;
		c.ipadx = 5; c.ipady = 5;
		c.gridx = 1; c.gridy = 2;
		containerProperties.add(GUIUtils.prepareLabel(collection.getConfiguration().getSingleProcess().getSingleActivityPercent()), c);
		
		c = new GridBagConstraints();
		c.anchor = GridBagConstraints.EAST;
		c.ipadx = 5; c.ipady = 5;
		c.gridx = 0; c.gridy = 3;
		containerProperties.add(GUIUtils.prepareLabel("Sequence percentage: ", SwingConstants.RIGHT), c);
		
		c = new GridBagConstraints();
		c.anchor = GridBagConstraints.WEST;
		c.ipadx = 5; c.ipady = 5;
		c.gridx = 1; c.gridy = 3;
		containerProperties.add(GUIUtils.prepareLabel(collection.getConfiguration().getSingleProcess().getSequencePercent()), c);
		
		c = new GridBagConstraints();
		c.anchor = GridBagConstraints.EAST;
		c.ipadx = 5; c.ipady = 5;
		c.gridx = 0; c.gridy = 4;
		containerProperties.add(GUIUtils.prepareLabel("AND split-join percentage: ", SwingConstants.RIGHT), c);
		
		c = new GridBagConstraints();
		c.anchor = GridBagConstraints.WEST;
		c.ipadx = 5; c.ipady = 5;
		c.gridx = 1; c.gridy = 4;
		containerProperties.add(GUIUtils.prepareLabel(collection.getConfiguration().getSingleProcess().getANDPercent()), c);
		
		c = new GridBagConstraints();
		c.anchor = GridBagConstraints.EAST;
		c.ipadx = 10; c.ipady = 10;
		c.gridx = 0; c.gridy = 5;
		containerProperties.add(GUIUtils.prepareLabel("XOR split-join percentage: ", SwingConstants.RIGHT), c);
		
		c = new GridBagConstraints();
		c.anchor = GridBagConstraints.WEST;
		c.ipadx = 10; c.ipady = 10;
		c.gridx = 1; c.gridy = 5;
		containerProperties.add(GUIUtils.prepareLabel(collection.getConfiguration().getSingleProcess().getXORPercent()), c);
		
		c = new GridBagConstraints();
		c.anchor = GridBagConstraints.EAST;
		c.ipadx = 5; c.ipady = 5;
		c.gridx = 0; c.gridy = 6;
		containerProperties.add(GUIUtils.prepareLabel("Maximum nested patterns: ", SwingConstants.RIGHT), c);
		
		c = new GridBagConstraints();
		c.anchor = GridBagConstraints.WEST;
		c.ipadx = 5; c.ipady = 5;
		c.gridx = 1; c.gridy = 6;
		containerProperties.add(GUIUtils.prepareLabel(collection.getConfiguration().getSingleProcess().getDeep()), c);
		
		c = new GridBagConstraints();
		c.anchor = GridBagConstraints.EAST;
		c.ipadx = 5; c.ipady = 5;
		c.gridx = 0; c.gridy = 7;
		containerProperties.add(GUIUtils.prepareLabel("Collection creation date: ", SwingConstants.RIGHT), c);
		
		c = new GridBagConstraints();
		c.anchor = GridBagConstraints.WEST;
		c.ipadx = 5; c.ipady = 5;
		c.gridx = 1; c.gridy = 7;
		containerProperties.add(GUIUtils.prepareLabel(new SimpleDateFormat("yyyy/MM/dd HH:mm").format(collection.getConfiguration().getCreationDate())), c);
		
		// list of processes
		DefaultListModel processesListModel = new DefaultListModel();
		for (PlgProcess p : collection) {
			processesListModel.addElement(p);
		}
		processesList = new JList(processesListModel);
		processesList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		processesList.setSelectionBackground(Color.darkGray);
		processesList.setSelectionForeground(Color.white);
		processesList.setBackground(Color.gray);
		
		JScrollPane processesScrollPane = new JScrollPane(processesList);
		SlickerDecorator.instance().decorate(processesScrollPane, Color.gray, Color.black, Color.darkGray);
		processesScrollPane.setBorder(new EmptyBorder(0, 0, 0, 0));
		buttonReloadView.addActionListener(new ProcessesListListener(containerProcess, processesList, context));
		
		JPanel tmp0 = new JPanel(); tmp0.setOpaque(false);
		tmp0.setLayout(new BorderLayout());
		tmp0.setBorder(new EmptyBorder(5, 0, 0, 0));
		tmp0.add(buttonReloadView, BorderLayout.EAST);
		
		containerList.add(processesScrollPane, BorderLayout.CENTER);
		containerList.add(tmp0, BorderLayout.SOUTH);
		
		// final layout
		
		containerMain.setLayout(new BorderLayout());
		containerMain.add(title, BorderLayout.NORTH);
		containerMain.add(containerBody, BorderLayout.CENTER);
		
		return containerMain;
	}
	
}
