package org.processmining.plugins.plg.io;

import java.io.IOException;
import java.io.InputStream;

import org.processmining.contexts.uitopia.annotations.UIImportPlugin;
import org.processmining.framework.abstractplugins.AbstractImportPlugin;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.plugins.plg.models.PLGProcessCollection;

/**
 * Class with the import plugin for a collection of processes
 * 
 * @author Andrea Burattin
 */
@Plugin(
	name = "Import PLG Process Collection (.plg)",
	returnLabels = { "Imported process collection" },
	returnTypes = { PLGProcessCollection.class },
	parameterLabels = { "File" },
	userAccessible = true
)
@UIImportPlugin(
	description = "PLG Process Collection (.plg)",
	extensions = { "plg" }
)
public class ImportProcessCollection extends AbstractImportPlugin {

	/**
	 * This plugin is used to import a process collection object
	 * 
	 * @param context
	 * @param input
	 * @param filename
	 * @param fileSizeInBytes
	 * @return
	 * @throws IOException 
	 */
	protected Object importFromStream(
			PluginContext context, 
			InputStream input, 
			String filename, 
			long fileSizeInBytes)
			throws Exception {
		context.log("Import started..."); 
		PLGProcessCollection p = PLGProcessCollection.importCollection(input);
		context.log("Import complete!");
		return p;
	}
}
