/**
 * 
 */
package org.processmining.plugins.plg.models;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;

import org.processmining.framework.annotations.AuthoredType;
import org.processmining.plugins.plg.config.BasicPluginConfiguration;

/**
 * 
 * @author Andrea Burattin
 */
@AuthoredType(
	typeName = "PLG Mining results",
	affiliation = BasicPluginConfiguration.AFFILIATION,
	author = BasicPluginConfiguration.AUTHOR,
	email = BasicPluginConfiguration.EMAIL
)
public class PLGMineResultCollection {
	
	private HashMap<PlgProcess, Set<PLGPair<PLGLog, Object>>> collection;
	private String name;
	
	
	/**
	 * Basic class constructor
	 * 
	 * @param name
	 * @param collectionTypes
	 */
	public PLGMineResultCollection(String name) {
		this.name = name;
		this.collection = new HashMap<PlgProcess, Set<PLGPair<PLGLog,Object>>>();
	}
	
	
	/**
	 * Method to get the mine result name
	 * 
	 * @return the name
	 */
	public String getName() {
		return name;
	}
	
	
	/**
	 * Method to add a result to the current collection
	 * 
	 * @param p the original process
	 * @param l the original log
	 * @param r the mined model
	 * @return the new set of pairs of logs and mined model associated with the
	 *         given process 
	 */
	public Object put(PlgProcess p, PLGLog l, Object r) {
		Set<PLGPair<PLGLog, Object>> s = collection.get(p);
		if (s == null) {
			s = new HashSet<PLGPair<PLGLog,Object>>();
			collection.put(p, s);
		}
		s.add(new PLGPair<PLGLog, Object>(l.getId(), l, r));
		return s;
	}


	/**
	 * This method returns the size of the collection
	 *
	 * @return the size of the collection
	 */
	public int size() {
		return collection.size();
	}


	/**
	 * This method to get the set of all the processes with a log and a miner
	 * result
	 *
	 * @return the set of processes
	 */
	public Set<PlgProcess> keySet() {
		return collection.keySet();
	}


	/**
	 * This method to get the set of all the logs for a given process
	 *
	 * @param p the given process
	 * @return the set of logs
	 */
	public Set<PLGPair<PLGLog, Object>> get(PlgProcess p) {
		return collection.get(p);
	}
}
